local L = LibStub("AceLocale-3.0"):GetLocale("ShinyStats")
if not L then return end

ShinyStats = LibStub("AceAddon-3.0"):NewAddon("ShinyStats")

local options = {
	name = "ShinyStats",
	handler = ShinyStats,
	type = 'group',
	args = {
		aceconfig = {
			name = L["ShinyStats settings (Ace style)"],
			desc = L["Shows the Ace configuration dialog for ShinyStats."],
			type = 'execute',
			func = function() LibStub("AceConfigDialog-3.0"):Open("ShinyStats") end,
			hidden = true,
			cmdHidden = false,
		},
		config = {
			name = L["ShinyStats settings"],
			desc = L["Shows the blizzard configuration dialog for ShinyStats."],
			type = 'execute',
			func = "ShinyConfig",
			hidden = true,
			cmdHidden = false,
		},
		conf = {
			name = L["ShinyStats settings"],
			desc = L["Alias for /shiny config."],
			type = 'execute',
			func = "ShinyConfig",
			hidden = true,
			cmdHidden = false,
		},
		settings = {
			name = L["ShinyStats settings"],
			desc = L["Alias for /shiny config."],
			type = 'execute',
			func = "ShinyConfig",
			hidden = true,
			cmdHidden = false,
		},
		reset = {
		name = L["Reset"],
		desc = L["Reset settings back to defaults."],
		type = 'execute',
		func = function() ShinyStats.db:ResetProfile() end,
		},
		shortenEnchants = {
			type = 'toggle',
			name = L["Shorten Enchants"],
			desc = L["Whether or not you wish for the addon to reduce enchant/socket texts to a minimum."],
			set = 'SetOption',
			get = 'GetOption',
		},
		prefixEnchants = {
			type = 'toggle',
			name = L["Enchant Prefixing"],
			desc = L["Whether or not you wish for the addon to prefix enchants."],
			set = 'SetOption',
			get = 'GetOption',
		},
		shortenEquips = {
			type = 'toggle',
			name = L["Shorten Equips"],
			desc = L["Whether or not you wish for the addon to reduce equip texts to a minimum."],
			disabled = true,
			set = 'SetOption',
			get = 'GetOption',
		},		
		shortenSeparator = {
			type = 'input',
			name = L["Shorten Separator"],
			desc = L["The string replacing substituted values. (Does nothing if ShortenEquips and ShortenEnchants are disabled)"],
			set = 'SetOption',
			get = 'GetOption',
		},
		colors = {
			type = 'group',
			name = L["Colors"],
			desc = L["Set the colors of the various item stats."],
			args = {}
		},
	},
}

local AceConfig = LibStub("AceConfig-3.0")
local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local BlizPanel = nil

AceConfig:RegisterOptionsTable("ShinyStats", options, {"shinystats", "shiny"})
BlizPanel = AceConfigDialog:AddToBlizOptions("ShinyStats", "ShinyStats")

function ShinyStats:ShinyConfig()
	InterfaceOptionsFrame_OpenToCategory(BlizPanel)
end

local defaults = {
	profile = {
		shortenEnchants = true,
		shortenEquips	= true,
		shortenSeparator = ", ",
		prefixEnchants 	= true,
		colors = {
			["agi"] = "ffee03", 	-- Agility
			["sta"] = "dda357", 	-- Stamina
			["spi"] = "b7ffb8", 	-- Spirit
			["int"] = "b395ff", 	-- Intellect
			["str"] = "ff000a", 	-- Strength
			
			["exp"] = "bbff00", 	-- Expertise
			["cri"] = "e8850d", 	-- Crit Rating
			["hit"] = "00ffc7", 	-- Hit Rating
			["has"] = "b3eeff", 	-- Haste Rating
			["ap"] = "ff494f", 		-- Attack Power
			
			["frr"] = "188bff", 	-- Frost Res
			["fr"] = "ff8300", 		-- Fire Res
			["nr"] = "87ff3c", 		-- Nature Res
			["sr"] = "bd33b1", 		-- Shadow Res
			["ar"] = "a0f2ff", 		-- Arcane Res
			
			["ilvl"] = "F2C038", 	-- Item Level
			
			["blo"] = "a2793c", 	-- Block
			["par"] = "f600ff", 	-- Parry
			["dod"] = "bcff00", 	-- Dodge
			["res"] = "686eff",		-- Resilience
			
			["dmg"] = "ff8930", 	-- Damage
			["dps"] = "ff96ad", 	-- DPS
			["spd"] = "fff250",		-- Speed
			["arm"] = "b9b9b9", 	-- Armor
			
			["sp"]	= "ff73ba",		-- Spell Power
			["mr"]	= "ff73ba",		-- Mastery Rating
			
			-- Specials
			["ench"] = "00ff00",	-- Ench: Color
			
			-- Unspecified
			['*']	 = "ffffff",	-- Default for unspecified
		},
	}
}

-- Bind colors to more descriptive names for options (Localization)
local colorNames = {
	-- Option Stat names
	["agi"] = L["Agility"],			-- Agility
	["sta"] = L["Stamina"],			-- Stamina
	["spi"] = L["Spirit"],			-- Spirit
	["int"] = L["Intellect"],			-- Intellect
	["str"] = L["Strength"],			-- Strength

	["exp"] = L["Expertise Rating"],			-- Expertise
	["cri"] = L["Critical Strike Rating"],			-- Crit Rating
	["hit"] = L["Hit Rating"],			-- Hit Rating
	["has"] = L["Haste Rating"],			-- Haste Rating
	["ap"] = L["Attack Power"],			-- Attack Power

	["frr"] = L["Frost Resistance"],			-- Frost Res
	["fr"] = L["Fire Resistance"],			-- Fire Res
	["nr"] = L["Nature Resistance"],			-- Nature Res
	["sr"] = L["Shadow Resistance"],			-- Shadow Res
	["ar"] = L["Arcane Resistance"],			-- Arcane Res

	["ilvl"] = L["Item Level"],			-- Item Level

	["blo"] = L["Block Rating"],			-- Block
	["par"] = L["Parry Rating"],			-- Parry
	["dod"] = L["Dodge Rating"],			-- Dodge
	["res"] = L["Resilience Rating"],			-- Resilience

	["dmg"] = L["Weapon Damage"],			-- Damage
	["dps"] = L["Weapon DPS"],			-- DPS
	["spd"] = L["Weapon Speed"],			-- Speed
	["arm"] = L["Armor"],				-- Armor

	["sp"] = L["Spell Power"],			-- Spell Power
	["mr"] = L["Mastery Rating"],			-- Mastery Rating

	-- Specials
	["ench"] = L["Enchant Color"],			-- Ench: Color
}

function ShinyStats:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("ShinyStatsDB", defaults)
	
	self.db.RegisterCallback(self, "OnProfileChanged", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileCopied", "RefreshConfig")
	self.db.RegisterCallback(self, "OnProfileReset", "RefreshConfig")
	
	ShinyStats:RefreshConfig()
	
	for k,v in pairs(self.db.profile.colors) do
		options.args.colors.args[k] = {
					type = 'color',
					name = colorNames[k],
					set = 'SetOption',
					get = 'GetOption', 
					}
	end
end

function ShinyStats:SetOption(info,value, ...)
	local statements = {
		shortenEnchants	= function() self.db.profile.shortenEnchants = value end,
		shortenEquips	= function() self.db.profile.shortenEquips = value end,
		shortenSeparator = function() self.db.profile.shortenSeparator = gsub(value,"[\"\']","") end,
		prefixEnchants 	= function() self.db.profile.prefixEnchants = value end,
	}
	if info[#info-1] == "colors" and type(value) == "number" then
		local r,g,b,a = value, ...
		self.db.profile.colors[info[#info]] = string.format("%02x%02x%02x",r*255,g*255,b*255)
		self.updateRequired = true
	elseif statements[info[#info]] then
		statements[info[#info]]()
		print(format("ShinyStats: %s set to %s!",info[#info], tostring(value)))
	end
end

function ShinyStats:Hex2Dec(hex)
	local dec,i = {}, 0
	for v in string.gmatch(hex,"(%x%x)") do
		i=i+1
		dec[i] = tonumber(v,16)/255
	end
	return dec[1],dec[2],dec[3]
end

function ShinyStats:GetOption(info)
	if info[#info-1] == "colors" then
		local r,g,b = ShinyStats:Hex2Dec(self.db.profile.colors[info[#info]])
		return r,g,b,1
	else
		return self.db.profile[info[#info]]
	end
end

function ShinyStats:RefreshConfig()
	-- Bind patterns to colors
	self.stats = nil
  	self.stats = {
		{L["%+?%d+ Agility"],	self.db.profile.colors.agi},													-- Agility
		{L["%+?%d+ Stamina"],	self.db.profile.colors.sta},													-- Stamina
		{L["%+?%d+ Spirit"],	self.db.profile.colors.spi},													-- Spirit
		{L["%+?%d+ Intellect"],	self.db.profile.colors.int},													-- Intellect
		{L["%+?%d+ Strength"],	self.db.profile.colors.str},													-- Strength
		
		{L["%+?%d+ Attack Power"],	self.db.profile.colors.ap},													-- Attack Power
		{L["Equip: Increases attack power by %d+%.*"], self.db.profile.colors.ap},								-- Attack Power
		{L["attack power by %d+%.*"], self.db.profile.colors.ap},												-- Attack Power		
						
		{L["Equip: Increases your expertise rating by %d+%.*"],	self.db.profile.colors.exp},					-- Expertise
		{L["expertise rating by %d+%.*"],	self.db.profile.colors.exp},										-- Expertise
		{L["%+?%d+ Expertise Rating"],	self.db.profile.colors.exp},											-- Expertise					
								
		{L["Equip: Increases your haste rating by %d+%.*"],	self.db.profile.colors.has},						-- Haste
		{L["haste rating by %d+%.*"],	self.db.profile.colors.has},											-- Haste
		{L["%+?%d+ Haste Rating"],	self.db.profile.colors.has},												-- Haste
						
		{L["Equip: Increases your hit rating by %d+%.*"],	self.db.profile.colors.hit},						-- Hit	
		{L["hit rating by %d+%.*"],	self.db.profile.colors.hit},												-- Hit
		{L["%+?%d+ Hit Rating"],	self.db.profile.colors.hit},												-- Hit						
						
		{L["Equip: Increases your critical strike rating by %d+%.*"],	self.db.profile.colors.cri},			-- Crit	
		{L["critical strike rating by %d+%.*"],	self.db.profile.colors.cri},									-- Crit	
		{L["%+?%d+ Critical Strike Rating"],	self.db.profile.colors.cri},									-- Crit
		{L["%+?%d+ Crit Rating"],	self.db.profile.colors.cri},												-- Crit	
						
		{L["Item Level %d*"],	self.db.profile.colors.ilvl},													-- Item Level
						
		{L["%+?%d+ Armor"],	self.db.profile.colors.arm},														-- Armor

		{L["Equip: Increases your parry rating by %d*%.*"], self.db.profile.colors.par},						-- Parry
		{L["parry rating by %d*%.*"], self.db.profile.colors.par},												-- Parry
		{L["%+?%d+ Parry"],	self.db.profile.colors.par},														-- Parry

		{L["Equip: Increases your block rating by %d*%.*"], self.db.profile.colors.blo},						-- Block
		{L["block rating by %d*%.*"], self.db.profile.colors.blo},												-- Block	
		{L["%+?%d+ Block"],	self.db.profile.colors.blo},														-- Block	

		{L["Equip: Increases your dodge rating by %d*%.*"],	self.db.profile.colors.dod},						-- Dodge
		{L["dodge rating by %d*%.*"], self.db.profile.colors.dod},												-- Dodge		
		{L["%+?%d+ Dodge Rating"],	self.db.profile.colors.dod},												-- Dodge
						
		{L["Equip: Increases your resilience rating by %d*%.*"],	self.db.profile.colors.res},				-- Resilience
		{L["resilience rating by %d*%.*"], self.db.profile.colors.res},											-- Resilience	
		{L["%+?%d+ Resilience Rating"],	self.db.profile.colors.res},											-- Resilience

		{L["%+?%d+ Fire Resistance"],	self.db.profile.colors.fr},												-- Fire Res
		{L["%+?%d+ Frost Resistance"],	self.db.profile.colors.frr},											-- Frost Res
		{L["%+?%d+ Nature Resistance"],	self.db.profile.colors.nr},												-- Nature Res
		{L["%+?%d+ Shadow Resistance"],	self.db.profile.colors.sr},												-- Shadow Res
		{L["%+?%d+ Arcane Resistance"],	self.db.profile.colors.ar},												-- Arcane Res
						
		{L["Equip: Increases spell power by %d*%.*"],	self.db.profile.colors.sp},								-- Spell Power
		{L["spell power by %d*%.*"],	self.db.profile.colors.sp},												-- Spell Power
		{L["%+?%d+ Spell Power"],	self.db.profile.colors.sp},													-- Spell Power
						
		{L["%(%d+%.*%d* damage per second%)"],	self.db.profile.colors.dps},									-- DPS
		
		{L["Speed %d+%.*%d*"],	self.db.profile.colors.spd},													-- Speed

		{L["Equip: Increases your mastery rating by %d*%.*"],	self.db.profile.colors.mr},						-- Mastery Rating
		{L["mastery rating by %d*%.*"],	self.db.profile.colors.mr},												-- Mastery Rating
		{L["%+?%d+ Mastery Rating"],	self.db.profile.colors.mr},												-- Mastery Rating
	}
	
	self.updateRequired = false
end

function ShinyStats:ExtendTooltip(tip, itemLink)
		--print(self.stats[1][2],self.db.profile.colors.agi)
		if self.updateRequired then ShinyStats:RefreshConfig() end
		local regions = {tip:GetRegions()}
		for i = 1, #regions do
			local region = regions[i]
			if region and region:GetObjectType() == "FontString" then
				local r,g = region:GetTextColor()
				local text = region:GetText()
				if text then
					local newText,m,mtext = nil, nil, nil
					
					-- Check if Socket bonus is activated (also check if you have reached the end part where no modification will be needed, then "continue")
					if (string.match(text,L["^Socket Bonus"]) and r < 0.51 and g < 0.51) or string.match(text,L["^<Shift"]) or string.match(text,L["^<Right"]) or string.match(text,L["^Set"]) then
						-- Continue
						-- Do nothing, stupid LUA lacks continue statements and a break kills the main loop
					else
						-- Add "Ench: " to enchants / Munch and's
						if not string.match(text,L["^Socket"]) and not string.match(text,L["^Equip"]) and not string.match(text,L["^Chance"]) and not string.match(text,L["^Set"]) and not string.match(text,L["^Use"]) then
							
							-- Add "Ench: " to enchants
							if r < 0.05 and g > 0.95 and self.db.profile.prefixEnchants then
								text = format("|cff%s%s: %s|r",self.db.profile.colors.ench,L["Ench"],text)
							end
							
							-- Munch 'ands' and replace with single space
							if self.db.profile.shortenEnchants then
								text = gsub(text,L["%s*and%s*"],self.db.profile.shortenSeparator)
							end
							if self.db.profile.shortenEquips then
								-- Do something later on in future
							end
						end
						
						----[[
						for i,v in ipairs(self.stats) do 
							-- Check if newText is already set (meaning there might be more matches on a single line)
							if newText ~= nil then
								mtext = newText
							else
								mtext = text
							end
							-- Perform matching against stats table
							newText,m = gsub(mtext,v[1],("|cff%s%s|r"):format(v[2],"%1"))
							
							if m > 0 then -- If any match found, set the text
								region:SetText(newText)
							end
						end
						----]]
					end
				end
			end
		end
		tip:Show()
end

local function hookTip(tooltip, method, action)
  if not tooltip then return end
  hooksecurefunc(tooltip, method, function(tip, ...)
    local link, count = action(...)
    if link then
      ShinyStats:ExtendTooltip(tip, link)
    end
  end)
end

local function hookCompareTip(tooltip)
  if not tooltip then return end
  hooksecurefunc(tooltip, 'SetHyperlinkCompareItem', function(tip, mainLink)
    local _, link = tip:GetItem()
    if link then
      ShinyStats:ExtendTooltip(tip, link)
    end
  end)
end

local function hookTipScript(tooltip)
  if tooltip and tooltip.HookScript then
    tooltip:HookScript('OnTooltipSetItem', function(tip, ...)
      local _, link = tip:GetItem()
      if link and GetItemInfo(link) then
        ShinyStats:ExtendTooltip(tip, link)
      end
    end)
  end
end

local hooked = false
local function HookTooltips()
  if hooked then return end
  hooked = true

  hookTipScript(GameTooltip)
  hookTipScript(ItemRefTooltip)

  hookCompareTip(ShoppingTooltip1)
  hookCompareTip(ShoppingTooltip2)
  hookCompareTip(ShoppingTooltip3)
  hookCompareTip(ItemRefShoppingTooltip1)
  hookCompareTip(ItemRefShoppingTooltip2)
  hookCompareTip(ItemRefShoppingTooltip3)

  if LinkWrangler and LinkWrangler.RegisterCallback then
    LinkWrangler.RegisterCallback("ShinyStats", function(frame,link)
      ShinyStats:ExtendTooltip(frame,link)
    end, "item")
  end
end

-- The following stuff is unfortunately necessary, because some addons *override*
-- the tooltip handlers instead of hooking them :(

local f = CreateFrame("Frame")
local entered = false
local elapsed = 0
local timerStarted = false
f:SetScript("OnEvent", function(f, e, ...) f[e](f, ...) end)

function f:PLAYER_ENTERING_WORLD()
	entered = true
	f:ScheduleHook()
end

function f:ADDON_LOADED()
	if entered then
		f:ScheduleHook()
	end
end

function f:ScheduleHook()
	if timerStarted then
		elapsed = 0
	else
		f:SetScript("OnUpdate", f.OnTimer)
	end
end

function f:OnTimer(eps)
	elapsed = elapsed + eps
	if elapsed >= 3 then
		f:SetScript("OnUpdate", nil)
		HookTooltips()
	end
end

f:RegisterEvent("PLAYER_ENTERING_WORLD")
f:RegisterEvent("ADDON_LOADED")

